import java.rmi.*;
import java.rmi.server.*;
import java.util.Vector;
import java.net.*;
import java.io.*;
import java.util.*;
import java.lang.*;
    

/**This class represents the Global Name Server that create the simulation world
  * and maintains the routing information for all the PEs.
  *@author Yin Xiong
  *@version 2.0(10/2006)
  *@since 2.0
  */
public class GNS extends UnicastRemoteObject implements GNSInterface{
	
/**the flag indicating the end of the simulation*/
Flag flag;

/**the current number of PEs registered with the GNS*/
int PEcount;

/**the table that contains the information about the PEs*/
Vector<PEInfo> PEtable;

/**the routing table*/
Vector<RoutingInfo> routingTable;

/**the total number of PEs for this simulation*/
int numPEs;

/**the total number of LPs for this simulation*/
int numLPs;

/**the vector that contains the LP configuration information obtained from a file*/
 Vector<LPConfigInfo> lpConfigInfoVector;

/**the number of LPs for each PE*/
int lpForEach;

/**the number of LPs for the first PE which can be different from other PEs*/
int lpForFirst;

/**Constructor*/
public GNS() throws RemoteException{
  super();	
  flag=new Flag();
  PEcount=-1;  	
  PEtable=new Vector<PEInfo>();
  lpConfigInfoVector=new Vector<LPConfigInfo>();
  routingTable=new Vector<RoutingInfo>();
  lpForEach=0;
  lpForFirst=0;
}

/**Gets the end of simulation flag.
  */
public int getEOSflag()
{
  return flag.get();
}//end getEOFflag


/**Sets the end of simulation flag, sends the flag to each PE and shuts down
  *the RMI server for each PE.
 *@param f int negative number indicating the end of simulation
 */
public void setEOSflag(int f)
{

    System.out.println("PE table size: " + PEtable.size());
    //dumpPEtable();

    Peer[] peers=new Peer[PEtable.size()];

   //sets end of sim flag
   flag.set(f);
   System.out.println("gns set eos flag");
   String targetName=null;

   //inform all the PEs that this is the end of sim
   for(int i=0;i<PEtable.size();i++){
    PEInfo pinfo=(PEInfo)PEtable.get(i);
    String dest=pinfo.machineName;
    peers[i]=null;
 
    while(peers[i]==null){
         try{
            targetName="rmi://" + dest + "/Peer";
            peers[i] =(Peer)Naming.lookup(targetName);
           }
         catch(Exception e){ 
            //e.printStackTrace();
            //System.out.println(myMachineName+ " trying to connect to " + dest);
          }
      }//end while

   try{
    if(peers[i]!=null){
      peers[i].setEOSflag(-1);
     }//end if
    }
  catch(Exception e){
    //e.printStackTrace();
    System.out.println("fail to set flag");
  } 

 }//end for        
   

  try{
     Thread thread=Thread.currentThread();
     thread.sleep(1000);
  }
  catch(Exception e){
    e.printStackTrace();
  }

  //kill peer servers
 try{
  for(int i=0;i<PEtable.size();i++){
   PEInfo pinfo=(PEInfo)PEtable.get(i);
   System.out.println("kill " + pinfo.machineName);
     if(peers[i]!=null)
       peers[i].shutDown();
  }//end for
 }
 catch(Exception e){
    //e.printStackTrace();
    System.out.println("fail to kill");

  } 
}//end setEOSflag

/**Dumps the PEtable; for debugging only*/
void dumpPEtable()
{
   for(int i=0;i<PEtable.size();i++){
     PEInfo pinfo=(PEInfo)PEtable.get(i);
     System.out.println(pinfo.toString());
   }

}//end dumpPEtable

/**Starts the GNS as a rmi server offering remote methods for the PEs.
  */
public void startGNS() throws RemoteException
{
  try{
     Naming.bind("rmi://localhost/GNS",this);
     System.out.println("GNS started successfully. Waiting for PEs...");
    }
		
 catch(Exception e) { 
      System.err.println("error binding server: "+e); 
      System.exit(1);}
}

/**Remote method that register a new PE and returns the LP configuration
  *information to the new PE.
  *@param pm String the machine name of the PE
  */
public Vector<LPConfigInfo> registerPE(String pm){

    String peip=null; 
    boolean isRegistered =false;
    int begin=-1;
    int end=-1;
    int pid=-1;
    PEInfo pinfo=null;

  //check if the PE is already registered
  for(int i=0;i<PEtable.size();i++){
    pinfo=(PEInfo)PEtable.get(i);
    if(pm.equalsIgnoreCase(pinfo.machineName)){
       isRegistered=true;
       break;
    }
  }//end for

   if(!isRegistered){             
    PEcount++;      
    pid=PEcount;
   try{
      InetAddress peAddress=InetAddress.getByName(pm);
      peip= peAddress.getHostAddress();
      pinfo=new PEInfo(pid, peip, pm, -1, -1);
      PEtable.add(pinfo);

      System.out.println("New Client accepted: "+ PEcount + " " + pm +  " " + peip);
     }
   catch(Exception e){
      e.printStackTrace();
   }//end catch
  }//end if

  else{
     peip=pinfo.PEip;
     begin=pinfo.begin;
     end=pinfo.end;
     pid=pinfo.PEid;
  }     

   Vector<LPConfigInfo> lpinfo=new Vector<LPConfigInfo>();
       if(pid>=0)
          lpinfo=getLPConfigInfo(isRegistered, pid, peip, pm, begin, end);

  // System.out.println("dumping routingTable: ");
   //dumpRoutingTable();
    //dumpPEtable();
    return lpinfo;
}

/**main function*/
public static void main (String [] args){

 Flag flag=new Flag();
 boolean endOfSim=false;
 int sleepTime=300;
 String myname=null;
 String fileName;
 int numPEs;
  
 int len,tasks,me,size,rc;
 me=size=0;
  
//get file name from command line
if(args.length<2){
  System.out.println("Usage: java sim [config file name] [# of PEs]");
  System.exit(1);
}       
        
else{
  fileName=args[0];
  //System.out.println("Config file: " +fileName + "\n");
    
  numPEs=Integer.parseInt(args[1]);
  //System.out.println("Number of PEs: " + numPEs+ "\n");
  
  if(args.length>2){
  sleepTime=Integer.parseInt(args[2]);
  //System.out.println("sleep time: " + sleepTime);
  }
  

 try{
  GNS gns=new GNS();
  gns.procConfigFile(fileName, numPEs);
  gns.startGNS();
 }
  
 catch(RemoteException e) { 
    System.out.println(e);
  }//edn catch
 }//end else
 }//end main

/**Prints out the information contained in the routing table; for debugging only
  */
void dumpRoutingTable()
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    System.out.println(rinfo.toString());
  }
}//end dumpRoutingTable


/**Looks up the routing info by an LP's application id.
  *@param appid String representing the application id of the LP.
  */
public RoutingInfo getRoutingInfoByAid(String appid)
{   
  //System.out.println("in gns, aid: " + appid);
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    //System.out.println("in gns, aid, rinfo.aid: " + appid + " " + rinfo.APPid);
    if(appid.equalsIgnoreCase(rinfo.APPid))

        return rinfo;
   }//end for
  return null;


}//end getReceiverByAid

/**Looks up the routing info by an LP id.              
  *@param lid int representing the LP id.                        
  */
public RoutingInfo getRoutingInfoByLid(int lid)      
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    if(lid==rinfo.LPid)                   
        return rinfo;
   }//end for 
  return null;
}//end getReceiverByLid

/**Processes the configuration file. The configuration file is text file. Each line
  *contains configuration information for an LP in the format of: <br>
  *applicationID  applicationName  className data <br>
  *@param configFile String indicating the name of the configuration file.
  *@param size int the number of PEs.
  */

public void procConfigFile(String configFile, int size)
{
     
  //read configuration file to decide how many instance for each class need to be created
     try{
        String newline=null;
        BufferedReader bf=new BufferedReader(new FileReader(configFile));
        StringTokenizer st;
        String id, name, className, dat;

        numPEs=size;
        numLPs=0;

        //each line is of the format: id name objName data
        while((newline=bf.readLine())!=null){
            //System.out.println(" " + newline);
            st=new StringTokenizer(newline, "\t");
            //System.out.println("num tokens: " + st.countTokens());
            if(st.countTokens()!=4){
              System.out.println("Malformated configration file!");
              System.exit(1);
            }//end if
      
            else{
                id= st.nextToken();
                id=id.trim();
        
                if(!id.equals("*")){
                   name=st.nextToken();
                   className=st.nextToken();
                   dat=st.nextToken();
                   LPConfigInfo lpcInfo=new LPConfigInfo(numLPs, id,name, className,dat);
                   lpConfigInfoVector.add(lpcInfo);
                   numLPs++;
                 }//end if the line is not comment
             }//end else if count of tokens !=4
     }//end while
        
         bf.close();
        
  }//end try

   catch(Exception e){
      e.printStackTrace();
   }
        
   //computes number of LPs for the first PE and for each of the rest
   int numLPs=lpConfigInfoVector.size();
   int rmder=numLPs%(numPEs-1);
   lpForEach=numLPs/(numPEs-1);
   lpForFirst=lpForEach+rmder;
   //System.out.println("in gns, numLPs,numPEs: " + numLPs+ " " + numPEs);
   //System.out.println("in gns,remainder, lpforeach, lpforfirst: " 
     //               + rmder + " " +lpForEach + " " + lpForFirst);
                          
}//end procConfigFile


/**Returns the LP configuration information to a newly-registered PE.
  *@param peid int the system-wide id for the PE
  *@param peip String the IP address of this PE
  *@param pm String the machine name of the PE
  *@param begin int the beginning point of the LP configuration information
  *@param end int the ending point of the LP configuration information
  */                            
Vector<LPConfigInfo> getLPConfigInfo(boolean isRegistered, int peid,String peip, 
                                     String pm, int begin, int end)
{
  if(!isRegistered){
     if(peid==0){
       begin=0;
       end=lpForFirst;
     }
     else{
       begin=(peid-1)*lpForEach + lpForFirst;
       end=begin+lpForEach;
    }     
  }

  System.out.println("for PE " + peid + " from, to: " + begin + " " + end);

  Vector<LPConfigInfo> v=new Vector<LPConfigInfo>();
  for(int i=begin; i<end; i++){
    LPConfigInfo lpinfo=(LPConfigInfo)lpConfigInfoVector.get(i);
    lpinfo.PEid=peid;
    v.add(lpinfo);
    if(!isRegistered){ 
       RoutingInfo rinfo=new RoutingInfo(peid, peip, pm, i,lpinfo.APPid);
       routingTable.add(rinfo);
     }//end if
   }
  return v;
}//end getLPConfigInfo

/**Returns the number of LPs for each PE.
  */                            
int getLPNumForEach()
{
   return lpForEach;
   
}//end getLPForNumEach


}//end class
