import java.io.*;
import java.util.*;
import java.lang.*;


/** This class represents the local name server that routes the messages among PEs. Each
  * PE has a related local name server.
  * @author Yin Xiong
  * @author Glenn Matthews
  * @version 1.2 (06/2005)
  */
  
public class LocalNameServer{

/**The ID of the PE this local name server serves.*/
int PEid;

/**The address of the global name server.*/
String gns;

/**A vector that stores the LP address information */
Vector<RoutingInfo> routingTable;

/**Construct a new LocalNameServer object.
  *@param pid int the system-assigned id of this PE
  */
public LocalNameServer(int pid)
{
   PEid=pid;
   routingTable=new Vector<RoutingInfo>();
}//end constructor





/**Displays the contents of the routing table.*/
public void dump()
{
   for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    rinfo.dump();
  }
}//end dumpRoutingTable

/**Looks up the routing info by an LP's application id.
  *@param appid String representing the application id of the LP.
  */
public RoutingInfo getRoutingInfoByAid(String appid)
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    if(appid.equalsIgnoreCase(rinfo.APPid))
        return rinfo;
   }//end for
  return null;
}//end getReceiverByAid

/**Looks up the routing info by an LP id.
  *@param lid int representing the LP id.
  */
public RoutingInfo getRoutingInfoByLid(int lid)
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    if(lid==rinfo.LPid)
        return rinfo;
   }//end for
  return null;
}//end getReceiverByLid

/**Adds a row of routing inforamtion into the routing table.
  *@param rinfo an RoutingInfo instance representing a row in the table.
  *@return the size of the routing table.
  */
public int addRoutingInfo(RoutingInfo rinfo)
{
   routingTable.add(rinfo);
   return routingTable.size();
}//end addRoutingRow


/**Looks up the PE id of an LP by its application id.
  *@param appid String representing the application id of the LP.
  */
public int getPidByAid(String appid)
{
  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    if(appid.equalsIgnoreCase(rinfo.getAPPid()))
        return rinfo.getPEid();
   }//end for
  return -4;
}//end getPidByAid

/**Looks up the LP id of an LP by its application id.
  *@param appid String representing the application id of the LP.
  */
public int getLidByAid(String appid)
{
  //System.out.println("LNS,routing table size: " + routingTable.size());

  for(int i=0;i<routingTable.size();i++){
    RoutingInfo rinfo=(RoutingInfo)routingTable.get(i);
    //System.out.println("in and out id: " + appid + " " + rinfo.getAPPid());

    if(appid.equalsIgnoreCase(rinfo.getAPPid()))
        return rinfo.getLPid();
   }//end for
  return -5;
}//end getLidByAid

/**Returns (as a tab-separated list) the APPids of every known LP. 
 * @since 1.2 */
public String getAPPids()
{
	String s = "";
	for (int i=0; i<routingTable.size(); i++) {
		RoutingInfo r = (RoutingInfo)routingTable.get(i);
		s += r.getAPPid() + '\t';
	}
	return s.trim();
}

}//end class LocalNameServer
