import java.rmi.*;
import java.rmi.server.*;
import java.io.*;
import java.net.*;
import java.util.concurrent.PriorityBlockingQueue;

/**This is a RMI-specific class. It creates a rmi server and runs it until it is
  *shut down from outside by the GNS server.
  *@author Yin Xiong
  *@version 2.0 (RMI version, 10/2006)
  *@since 2.0
  */
public class MsgReceiver extends UnicastRemoteObject implements Peer, Runnable{

/**the queue that contains the incoming message and is shared with the 
  *SchedulingLoop
  */
PriorityBlockingQueue<Message> msgQueue;

/**the IP address of the machine on which this rmi server is running*/
String myip;

/**the name of the machine on which this rmi server is running*/
String myMachineName;

/**the end of simulation flag that is shared by the rmi sender thread, the
  *rmi receiver thread and the SchedulingLoop thread.
  */
Flag flag;


/**constructs a MsgReceiver object.
  *@param mq a PriorityBlockingQueue that contains the incoming message and is
  *shared with the SchedulingLoop
  *@param f a Flag object indicating the end of simulation and is shared among
  *three threads running on this PE.
  */
public MsgReceiver(PriorityBlockingQueue<Message> mq, Flag f)throws RemoteException{
  super();
  flag=f;
  msgQueue=mq;
  //System.out.println(myMachineName+ " in receiver, msgQ size: " + msgQueue.size());

try{
  InetAddress address=InetAddress.getLocalHost();
  myMachineName=address.getHostName();
  myip=address.getHostAddress();
 }
catch(Exception e){
  e.printStackTrace();
}
  
}

/**Runs this thread. It's mainly set up the rmi server offering remote method
  *receiveMsg(Message msg). The server runs until somebody manually shuts it down
  *or by invoking UnicastRemoteObject's <i>unexportObject</i> method
  *
  */
public void run()
{
   try{
     Naming.bind("rmi://localhost/Peer",this);
     System.out.println("PE server started successfully on " + myMachineName + " " + myip);
    }

   catch(Exception e) {
     System.err.println("error binding server: "+e);
     System.exit(1);
   }

   	
}//end run


/**Sets the end of sim flag so any PE that enquires about the end of simulation will
  *get the same value.
  *@param f int the value to be set to the flag. Negative value indicates the end of
  *simulation.
  */
public void setEOSflag(int f)
{
   flag.set(f);

}//end setEOSflag


/**Remotely shuts down the rmi servers running on the PEs.
  */
public void shutDown()
{
 try{
  System.out.println(myMachineName + " shutting down...");
  unexportObject(this, true);
  }
 catch(Exception e){
   e.printStackTrace();
 }

}//end shutDown


/**Receives messages. The message to be sent is send as the input parameter to a 
  *remote method.
  *@param message a Message object that is to be sent to a PE.
  */
public int receiveMsg(Message message)
{
   
   msgQueue.offer(message);
   //System.out.println(myMachineName + " received " + message.getContent());
   return 1;
}
	
}//end class
